@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import calendar
 import json
 import mock
 import time
@@ -8,8 +9,8 @@ import unittest
 from datetime import datetime
 from intercom.collection_proxy import CollectionProxy
 from intercom.lib.flat_store import FlatStore
-from intercom import Intercom
-from intercom import User
+from intercom.client import Client
+from intercom.user import User
 from intercom import MultipleMatchingUsersError
 from intercom.utils import create_class_instance
 from mock import patch
@@ -19,26 +20,30 @@ from nose.tools import ok_
 from nose.tools import istest
 from tests.unit import get_user
 from tests.unit import mock_response
+from tests.unit import page_of_users
 
 
 class UserTest(unittest.TestCase):
 
+    def setUp(self):
+        self.client = Client()
+
     @istest
     def it_to_dict_itself(self):
         created_at = datetime.utcnow()
         user = User(
             email="jim@example.com", user_id="12345",
             created_at=created_at, name="Jim Bob")
-        as_dict = user.to_dict
+        as_dict = user.to_dict()
         eq_(as_dict["email"], "jim@example.com")
         eq_(as_dict["user_id"], "12345")
-        eq_(as_dict["created_at"], time.mktime(created_at.timetuple()))
+        eq_(as_dict["created_at"], calendar.timegm(created_at.utctimetuple()))
         eq_(as_dict["name"], "Jim Bob")
 
     @istest
     def it_presents_created_at_and_last_impression_at_as_datetime(self):
         now = datetime.utcnow()
-        now_ts = time.mktime(now.timetuple())
+        now_ts = calendar.timegm(now.utctimetuple())
         user = User.from_api(
             {'created_at': now_ts, 'last_impression_at': now_ts})
         self.assertIsInstance(user.created_at, datetime)
@@ -60,9 +65,9 @@ class UserTest(unittest.TestCase):
         eq_('Joe Schmoe', user.name)
         eq_('the-app-id', user.app_id)
         eq_(123, user.session_count)
-        eq_(1401970114, time.mktime(user.created_at.timetuple()))
-        eq_(1393613864, time.mktime(user.remote_created_at.timetuple()))
-        eq_(1401970114, time.mktime(user.updated_at.timetuple()))
+        eq_(1401970114, calendar.timegm(user.created_at.utctimetuple()))
+        eq_(1393613864, calendar.timegm(user.remote_created_at.utctimetuple()))
+        eq_(1401970114, calendar.timegm(user.updated_at.utctimetuple()))
 
         Avatar = create_class_instance('Avatar')  # noqa
         Company = create_class_instance('Company')  # noqa
@@ -79,14 +84,14 @@ class UserTest(unittest.TestCase):
         eq_('bbbbbbbbbbbbbbbbbbbbbbbb', user.companies[0].id)
         eq_('the-app-id', user.companies[0].app_id)
         eq_('Company 1', user.companies[0].name)
-        eq_(1390936440, time.mktime(
-            user.companies[0].remote_created_at.timetuple()))
-        eq_(1401970114, time.mktime(
-            user.companies[0].created_at.timetuple()))
-        eq_(1401970114, time.mktime(
-            user.companies[0].updated_at.timetuple()))
-        eq_(1401970113, time.mktime(
-            user.companies[0].last_request_at.timetuple()))
+        eq_(1390936440, calendar.timegm(
+            user.companies[0].remote_created_at.utctimetuple()))
+        eq_(1401970114, calendar.timegm(
+            user.companies[0].created_at.utctimetuple()))
+        eq_(1401970114, calendar.timegm(
+            user.companies[0].updated_at.utctimetuple()))
+        eq_(1401970113, calendar.timegm(
+            user.companies[0].last_request_at.utctimetuple()))
         eq_(0, user.companies[0].monthly_spend)
         eq_(0, user.companies[0].session_count)
         eq_(1, user.companies[0].user_count)
@@ -121,22 +126,24 @@ class UserTest(unittest.TestCase):
             'update_last_request_at': True,
             'custom_attributes': {}
         }
-        with patch.object(Intercom, 'post', return_value=payload) as mock_method:
-            User.create(user_id='1224242', update_last_request_at=True)
+        with patch.object(Client, 'post', return_value=payload) as mock_method:
+            self.client.users.create(
+                user_id='1224242', update_last_request_at=True)
             mock_method.assert_called_once_with(
-                '/users/', update_last_request_at=True, user_id='1224242')
+                '/users/',
+                {'update_last_request_at': True, 'user_id': '1224242'})
 
     @istest
     def it_allows_easy_setting_of_custom_data(self):
         now = datetime.utcnow()
-        now_ts = time.mktime(now.timetuple())
+        now_ts = calendar.timegm(now.utctimetuple())
 
         user = User()
         user.custom_attributes["mad"] = 123
         user.custom_attributes["other"] = now_ts
         user.custom_attributes["thing"] = "yay"
         attrs = {"mad": 123, "other": now_ts, "thing": "yay"}
-        eq_(user.to_dict["custom_attributes"], attrs)
+        eq_(user.to_dict()["custom_attributes"], attrs)
 
     @istest
     def it_allows_easy_setting_of_multiple_companies(self):
@@ -146,7 +153,7 @@ class UserTest(unittest.TestCase):
             {"name": "Test", "company_id": "9"},
         ]
         user.companies = companies
-        eq_(user.to_dict["companies"], companies)
+        eq_(user.to_dict()["companies"], companies)
 
     @istest
     def it_rejects_nested_data_structures_in_custom_attributes(self):
@@ -166,16 +173,22 @@ class UserTest(unittest.TestCase):
 
     @istest
     def it_fetches_a_user(self):
-        with patch.object(Intercom, 'get', return_value=get_user()) as mock_method:  # noqa
-            user = User.find(email='somebody@example.com')
+        with patch.object(Client, 'get', return_value=get_user()) as mock_method:  # noqa
+            user = self.client.users.find(email='somebody@example.com')
             eq_(user.email, 'bob@example.com')
             eq_(user.name, 'Joe Schmoe')
-            mock_method.assert_called_once_with('/users', email='somebody@example.com')  # noqa
+            mock_method.assert_called_once_with(
+                '/users', {'email': 'somebody@example.com'})  # noqa
+
+    @istest
+    def it_gets_users_by_tag(self):
+        with patch.object(Client, 'get', return_value=page_of_users(False)) as mock_method:
+            users = self.client.users.by_tag(124)
+            for user in users:
+                ok_(hasattr(user, 'avatar'))
 
     @istest
-    # @httpretty.activate
     def it_saves_a_user_always_sends_custom_attributes(self):
-        user = User(email="jo@example.com", user_id="i-1224242")
 
         body = {
             'email': 'jo@example.com',
@@ -183,21 +196,18 @@ class UserTest(unittest.TestCase):
             'custom_attributes': {}
         }
 
-        with patch.object(Intercom, 'post', return_value=body) as mock_method:
-            user.save()
+        with patch.object(Client, 'post', return_value=body) as mock_method:
+            user = User(email="jo@example.com", user_id="i-1224242")
+            self.client.users.save(user)
             eq_(user.email, 'jo@example.com')
             eq_(user.custom_attributes, {})
             mock_method.assert_called_once_with(
                 '/users',
-                email="jo@example.com", user_id="i-1224242",
-                custom_attributes={})
+                {'email': "jo@example.com", 'user_id': "i-1224242",
+                 'custom_attributes': {}})
 
     @istest
     def it_saves_a_user_with_a_company(self):
-        user = User(
-            email="jo@example.com", user_id="i-1224242",
-            company={'company_id': 6, 'name': 'Intercom'})
-
         body = {
             'email': 'jo@example.com',
             'user_id': 'i-1224242',
@@ -206,21 +216,21 @@ class UserTest(unittest.TestCase):
                 'name': 'Intercom'
             }]
         }
-        with patch.object(Intercom, 'post', return_value=body) as mock_method:
-            user.save()
+        with patch.object(Client, 'post', return_value=body) as mock_method:
+            user = User(
+                email="jo@example.com", user_id="i-1224242",
+                company={'company_id': 6, 'name': 'Intercom'})
+            self.client.users.save(user)
             eq_(user.email, 'jo@example.com')
             eq_(len(user.companies), 1)
             mock_method.assert_called_once_with(
                 '/users',
-                email="jo@example.com", user_id="i-1224242",
-                company={'company_id': 6, 'name': 'Intercom'},
-                custom_attributes={})
+                {'email': "jo@example.com", 'user_id': "i-1224242",
+                 'company': {'company_id': 6, 'name': 'Intercom'},
+                 'custom_attributes': {}})
 
     @istest
     def it_saves_a_user_with_companies(self):
-        user = User(
-            email="jo@example.com", user_id="i-1224242",
-            companies=[{'company_id': 6, 'name': 'Intercom'}])
         body = {
             'email': 'jo@example.com',
             'user_id': 'i-1224242',
@@ -229,15 +239,18 @@ class UserTest(unittest.TestCase):
                 'name': 'Intercom'
             }]
         }
-        with patch.object(Intercom, 'post', return_value=body) as mock_method:
-            user.save()
+        with patch.object(Client, 'post', return_value=body) as mock_method:
+            user = User(
+                email="jo@example.com", user_id="i-1224242",
+                companies=[{'company_id': 6, 'name': 'Intercom'}])
+            self.client.users.save(user)
             eq_(user.email, 'jo@example.com')
             eq_(len(user.companies), 1)
             mock_method.assert_called_once_with(
                 '/users',
-                email="jo@example.com", user_id="i-1224242",
-                companies=[{'company_id': 6, 'name': 'Intercom'}],
-                custom_attributes={})
+                {'email': "jo@example.com", 'user_id': "i-1224242",
+                 'companies': [{'company_id': 6, 'name': 'Intercom'}],
+                 'custom_attributes': {}})
 
     @istest
     def it_can_save_a_user_with_a_none_email(self):
@@ -253,23 +266,23 @@ class UserTest(unittest.TestCase):
                 'name': 'Intercom'
             }]
         }
-        with patch.object(Intercom, 'post', return_value=body) as mock_method:
-            user.save()
+        with patch.object(Client, 'post', return_value=body) as mock_method:
+            self.client.users.save(user)
             ok_(user.email is None)
             eq_(user.user_id, 'i-1224242')
             mock_method.assert_called_once_with(
                 '/users',
-                email=None, user_id="i-1224242",
-                companies=[{'company_id': 6, 'name': 'Intercom'}],
-                custom_attributes={})
+                {'email': None, 'user_id': "i-1224242",
+                 'companies': [{'company_id': 6, 'name': 'Intercom'}],
+                 'custom_attributes': {}})
 
     @istest
     def it_deletes_a_user(self):
         user = User(id="1")
-        with patch.object(Intercom, 'delete', return_value={}) as mock_method:
-            user = user.delete()
+        with patch.object(Client, 'delete', return_value={}) as mock_method:
+            user = self.client.users.delete(user)
             eq_(user.id, "1")
-            mock_method.assert_called_once_with('/users/1/')
+            mock_method.assert_called_once_with('/users/1', {})
 
     @istest
     def it_can_use_user_create_for_convenience(self):
@@ -278,10 +291,11 @@ class UserTest(unittest.TestCase):
             'user_id': 'i-1224242',
             'custom_attributes': {}
         }
-        with patch.object(Intercom, 'post', return_value=payload) as mock_method:  # noqa
-            user = User.create(email="jo@example.com", user_id="i-1224242")
-            eq_(payload, user.to_dict)
-            mock_method.assert_called_once_with('/users/', email="jo@example.com", user_id="i-1224242")  # noqa
+        with patch.object(Client, 'post', return_value=payload) as mock_method:  # noqa
+            user = self.client.users.create(email="jo@example.com", user_id="i-1224242")  # noqa
+            eq_(payload, user.to_dict())
+            mock_method.assert_called_once_with(
+                '/users/', {'email': "jo@example.com", 'user_id': "i-1224242"})  # noqa
 
     @istest
     def it_updates_the_user_with_attributes_set_by_the_server(self):
@@ -291,10 +305,12 @@ class UserTest(unittest.TestCase):
             'custom_attributes': {},
             'session_count': 4
         }
-        with patch.object(Intercom, 'post', return_value=payload) as mock_method:
-            user = User.create(email="jo@example.com", user_id="i-1224242")
-            eq_(payload, user.to_dict)
-            mock_method.assert_called_once_with('/users/', email="jo@example.com", user_id="i-1224242")  # noqa
+        with patch.object(Client, 'post', return_value=payload) as mock_method:  # noqa
+            user = self.client.users.create(email="jo@example.com", user_id="i-1224242")  # noqa
+            eq_(payload, user.to_dict())
+            mock_method.assert_called_once_with(
+                '/users/',
+                {'email': "jo@example.com", 'user_id': "i-1224242"})  # noqa
 
     @istest
     def it_allows_setting_dates_to_none_without_converting_them_to_0(self):
@@ -303,10 +319,10 @@ class UserTest(unittest.TestCase):
             'custom_attributes': {},
             'remote_created_at': None
         }
-        with patch.object(Intercom, 'post', return_value=payload) as mock_method:
-            user = User.create(email="jo@example.com", remote_created_at=None)
+        with patch.object(Client, 'post', return_value=payload) as mock_method:
+            user = self.client.users.create(email="jo@example.com", remote_created_at=None)  # noqa
             ok_(user.remote_created_at is None)
-            mock_method.assert_called_once_with('/users/', email="jo@example.com", remote_created_at=None)  # noqa
+            mock_method.assert_called_once_with('/users/', {'email': "jo@example.com", 'remote_created_at': None})  # noqa
 
     @istest
     def it_gets_sets_rw_keys(self):
@@ -317,14 +333,14 @@ class UserTest(unittest.TestCase):
             'name': 'Bob Smith',
             'last_seen_ip': '1.2.3.4',
             'last_seen_user_agent': 'ie6',
-            'created_at': time.mktime(created_at.timetuple())
+            'created_at': calendar.timegm(created_at.utctimetuple())
         }
         user = User(**payload)
         expected_keys = ['custom_attributes']
         expected_keys.extend(list(payload.keys()))
-        eq_(sorted(expected_keys), sorted(user.to_dict.keys()))
+        eq_(sorted(expected_keys), sorted(user.to_dict().keys()))
         for key in list(payload.keys()):
-            eq_(payload[key], user.to_dict[key])
+            eq_(payload[key], user.to_dict()[key])
 
     @istest
     def it_will_allow_extra_attributes_in_response_from_api(self):
@@ -333,16 +349,10 @@ class UserTest(unittest.TestCase):
 
     @istest
     def it_returns_a_collectionproxy_for_all_without_making_any_requests(self):
-        with mock.patch('intercom.Request.send_request_to_path', new_callable=mock.NonCallableMock):  # noqa
-            res = User.all()
+        with mock.patch('intercom.request.Request.send_request_to_path', new_callable=mock.NonCallableMock):  # noqa
+            res = self.client.users.all()
             self.assertIsInstance(res, CollectionProxy)
 
-    @istest
-    def it_returns_the_total_number_of_users(self):
-        with mock.patch.object(User, 'count') as mock_count:
-            mock_count.return_value = 100
-            eq_(100, User.count())
-
     @istest
     def it_raises_a_multiple_matching_users_error_when_receiving_a_conflict(self):  # noqa
         payload = {
@@ -361,7 +371,7 @@ class UserTest(unittest.TestCase):
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
             with assert_raises(MultipleMatchingUsersError):
-                Intercom.get('/users')
+                self.client.get('/users', {})
 
     @istest
     def it_handles_accented_characters(self):
@@ -373,7 +383,7 @@ class UserTest(unittest.TestCase):
         resp = mock_response(content)
         with patch('requests.request') as mock_method:
             mock_method.return_value = resp
-            user = User.find(email='bob@example.com')
+            user = self.client.users.find(email='bob@example.com')
             try:
                 # Python 2
                 eq_(unicode('Jóe Schmö', 'utf-8'), user.name)
@@ -385,6 +395,8 @@ class UserTest(unittest.TestCase):
 class DescribeIncrementingCustomAttributeFields(unittest.TestCase):
 
     def setUp(self):  # noqa
+        self.client = Client()
+
         created_at = datetime.utcnow()
         params = {
             'email': 'jo@example.com',
@@ -401,28 +413,28 @@ class DescribeIncrementingCustomAttributeFields(unittest.TestCase):
     @istest
     def it_increments_up_by_1_with_no_args(self):
         self.user.increment('mad')
-        eq_(self.user.to_dict['custom_attributes']['mad'], 124)
+        eq_(self.user.to_dict()['custom_attributes']['mad'], 124)
 
     @istest
     def it_increments_up_by_given_value(self):
         self.user.increment('mad', 4)
-        eq_(self.user.to_dict['custom_attributes']['mad'], 127)
+        eq_(self.user.to_dict()['custom_attributes']['mad'], 127)
 
     @istest
     def it_increments_down_by_given_value(self):
         self.user.increment('mad', -1)
-        eq_(self.user.to_dict['custom_attributes']['mad'], 122)
+        eq_(self.user.to_dict()['custom_attributes']['mad'], 122)
 
     @istest
     def it_can_increment_new_custom_data_fields(self):
         self.user.increment('new_field', 3)
-        eq_(self.user.to_dict['custom_attributes']['new_field'], 3)
+        eq_(self.user.to_dict()['custom_attributes']['new_field'], 3)
 
     @istest
     def it_can_call_increment_on_the_same_key_twice_and_increment_by_2(self):  # noqa
         self.user.increment('mad')
         self.user.increment('mad')
-        eq_(self.user.to_dict['custom_attributes']['mad'], 125)
+        eq_(self.user.to_dict()['custom_attributes']['mad'], 125)
 
     @istest
     def it_can_save_after_increment(self):  # noqa
@@ -438,9 +450,102 @@ class DescribeIncrementingCustomAttributeFields(unittest.TestCase):
                 'name': 'Intercom'
             }]
         }
-        with patch.object(Intercom, 'post', return_value=body) as mock_method:  # noqa
+        with patch.object(Client, 'post', return_value=body) as mock_method:  # noqa
             user.increment('mad')
-            eq_(user.to_dict['custom_attributes']['mad'], 1)
-            user.save()
-            ok_('email' not in user.identity_hash)
-            ok_('user_id' in user.identity_hash)
+            eq_(user.to_dict()['custom_attributes']['mad'], 1)
+            self.client.users.save(user)
+
+
+class DescribeBulkOperations(unittest.TestCase):  # noqa
+
+    def setUp(self):  # noqa
+        self.client = Client()
+
+        self.job = {
+            "app_id": "app_id",
+            "id": "super_awesome_job",
+            "created_at": 1446033421,
+            "completed_at": 1446048736,
+            "closing_at": 1446034321,
+            "updated_at": 1446048736,
+            "name": "api_bulk_job",
+            "state": "completed",
+            "links": {
+                "error": "https://api.intercom.io/jobs/super_awesome_job/error",
+                "self": "https://api.intercom.io/jobs/super_awesome_job"
+            },
+            "tasks": [
+                {
+                    "id": "super_awesome_task",
+                    "item_count": 2,
+                    "created_at": 1446033421,
+                    "started_at": 1446033709,
+                    "completed_at": 1446033709,
+                    "state": "completed"
+                }
+            ]
+        }
+
+        self.bulk_request = {
+            "items": [
+                {
+                    "method": "post",
+                    "data_type": "user",
+                    "data": {
+                        "user_id": 25,
+                        "email": "alice@example.com"
+                    }
+                },
+                {
+                    "method": "delete",
+                    "data_type": "user",
+                    "data": {
+                        "user_id": 26,
+                        "email": "bob@example.com"
+                    }
+                }
+            ]
+        }
+
+        self.users_to_create = [
+            {
+                "user_id": 25,
+                "email": "alice@example.com"
+            }
+        ]
+
+        self.users_to_delete = [
+            {
+                "user_id": 26,
+                "email": "bob@example.com"
+            }
+        ]
+
+        created_at = datetime.utcnow()
+        params = {
+            'email': 'jo@example.com',
+            'user_id': 'i-1224242',
+            'custom_attributes': {
+                'mad': 123,
+                'another': 432,
+                'other': time.mktime(created_at.timetuple()),
+                'thing': 'yay'
+            }
+        }
+        self.user = User(**params)
+
+    @istest
+    def it_submits_a_bulk_job(self):  # noqa
+        with patch.object(Client, 'post', return_value=self.job) as mock_method:  # noqa
+            self.client.users.submit_bulk_job(
+                create_items=self.users_to_create, delete_items=self.users_to_delete)
+            mock_method.assert_called_once_with('/bulk/users', self.bulk_request)
+
+    @istest
+    def it_adds_users_to_an_existing_bulk_job(self):  # noqa
+        self.bulk_request['job'] = {'id': 'super_awesome_job'}
+        with patch.object(Client, 'post', return_value=self.job) as mock_method:  # noqa
+            self.client.users.submit_bulk_job(
+                create_items=self.users_to_create, delete_items=self.users_to_delete,
+                job_id='super_awesome_job')
+            mock_method.assert_called_once_with('/bulk/users', self.bulk_request)
